/**
    @file ft51_dfu.h

    @brief Library header for DFU (Device Firmware Update) API.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_DFU_H__
#define __FT51_DFU_H__

/**
    @def DFU_MAX_BLOCK_SIZE
    @brief Set the maximum block size for a packet of data from
     the host in a DFU_DNLOAD request.
 */
#define DFU_MAX_BLOCK_SIZE 64

/**
    @def DFU_MTP_TOP_ADDRESS
    @brief The topmost 64 bytes of the MTP are reserved. This
     specifies the first address reserved in the MTP on the
     FT51 to prevent overwriting the reserved block.
 */
#define DFU_MTP_TOP_ADDRESS (0x4000 - 64)

/**
    @def DFU_TIMEOUT
    @brief Suggested timeout for DFU downloads.
 */
#define DFU_TIMEOUT 0x2000

/**
 @brief     Determine current mode of DFU
 @return    Returns non-zero if the DFU state machine is in
            runtime mode.
 **/
unsigned char dfu_is_runtime(void);

/**
 @brief     Determine if DFU waiting to reset
 @return    Returns non-zero if the DFU state machine is in
            dfuMANIFEST-WAIT-RESET and is therefore waiting
            for a host reset or detach/attach sequence.
 **/
unsigned char dfu_is_wait_reset(void);

/**
 @brief      USB class request handler for DFU_DETACH

 @details    Move state machine to DFUSTATE_APPDETACH from
             DFUSTATE_APPIDLE and initialise a timeout within which
             time the host should set a USB reset on the bus.
             An ACK packet is sent on the USB control IN endpoint
             to the host to acknowledge successful completion of
             this request.
             @param[in]  timeout - number of milliseconds timeout before reverting
             to DFUSTATE_APPIDLE if no USB reset is forthcoming from
             the host.
 **/
void dfu_class_req_detach(uint16_t timeout);

/**
 @brief      USB class request handler for DFU_DNLOAD
 @details    Receive blocks of firmware from the host on the control
             OUT endpoint and program these into the MTP.
             If state machine is in DFUSTATE_DFUIDLE then move to
             DFUSTATE_DNLOAD_IDLE.
             If zero length data is received indicating the end of
             the firmware then move state machine to
             DFUSTATE_MANIFEST_WAIT_RESET.
             If an address or data length error are detected then
             move to the DFUSTATE_DFUERROR state.
             An ACK packet is sent on the USB control IN endpoint
             to the host to acknowledge successful completion of
             this request.
 @param[in]  address - starting address of data to program.
             It is up to the calling program to make sure this
             is calculated correctly.
 @param[in]  length - Number of bytes to program. This can be a
             maximum of DFU_MAX_BLOCK_SIZE.
 **/
void dfu_class_req_download(uint16_t block, uint16_t length);

/**
 @brief      USB class request handler for DFU_GETSTATUS
 @details    Return a structure to the host containing the
             current DFU state machine and status bytes. These
             are used by the application on the host to work out
             whether any errors have occurred and what the
             status of the device is.
             The structure is written via the control IN
             endpoint to the host.
 **/
void dfu_class_req_getstatus(void);

/**
 @brief      USB class request handler for DFU_CLRSTATUS
 @details    Clears an error state for the DFU state machine.
 **/
void dfu_class_req_clrstatus(void);

/**
 @brief      USB class request handler for DFU_ABORT
 @details    Aborts transaction and resets the DFU state machine.
 **/
void dfu_class_req_abort(void);

/**
 @brief      USB class request handler for DFU_GETSTATE
 @details    Return a single byte to the host containing the
             current DFU state machine byte.
             The data is written via the control IN endpoint
             to the host.
 **/
void dfu_class_req_getstate(void);

/**
 @brief      Implementation of USB reset state handler for DFU.
 @details    Move DFU state machine on when a USB reset is
             encountered.
             Return a  byte to the host indicating if the next state
             change of the DFU state machine byte requires code
             to be reloaded and run. i.e. a new program needs to
             be run.
 @return     status - non-zero if new program is to be run.
 **/
unsigned char dfu_reset(void);

/**
 @brief      Decrements detach_counter and adjusts state accordingly.
 @details    Moves to DFU_IDLE state if we have been in the DFU_APPDETACH
             state for too long.
             This is run from INTERRUPT LEVEL as a handler for an ISR.
 @return     Zero if timer running, non-zero if timer expired.
 **/
unsigned char dfu_timer(void);

#endif // __FT51_DFU_H__