/**
    @file ft51_usb_tmc.h

    @brief USBTMC FT51 header file

    USB Test and Measurement device class.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_USB_TMC_H__
#define __FT51_USB_TMC_H__

#include <stdint.h>
#include "ft51.h"

enum enum_msg_proc_state
{
	eIdle,
	eDependentMsg,
	eRequestResponse
};

/**
    @name   USBTMC Request Codes
    @brief  Table 15 Summary of USBTMC Class-Specific Requests
            Definitions of USBTMC request codes from:
             Universal Serial Bus Test and Measurement Class Specification (USBTMC)
 */
//@{
///sec 4.2.1 The  USBTMC Get Capabilities Request
enum {
	USBTMC_INITIATE_ABORT_BULK_OUT 		=	0x01,
	USBTMC_CHECK_ABORT_BULK_OUT_STATUS	=	0x02,
	USBTMC_INITIATE_ABORT_BULK_IN		=	0x03,
	USBTMC_CHECK_ABORT_BULK_IN_STATUS	=	0x04,
	USBTMC_INITIATE_CLEAR				=	0x05,
	USBTMC_CHECK_CLEAR_STATUS			=	0x06,
	USBTMC_GET_CAPABILITIES 			=	0x07,
};

// usbtmc Interface capabilities flag masks
#define USBTMC_INDICATOR_PULSE_REQ_MASK 0x04
#define USBTMC_TALK_ONLY_MASK			0x02
#define USBTMC_LISTEN_ONLY_MASK			0x01

// USBTMC Device capabilities flag mask
#define USBTMC_BULK_IN_TERMINATION		0X01

/**
    @name   USBTMC Status Codes
    @brief  Table 16 Summary of USBTMC Class-Specific Statuses
            Definitions of USBTMC status codes from:
             Universal Serial Bus Test and Measurement Class Specification (USBTMC)
 */
//@{
///sec 4.2.1 The  USBTMC Get Capabilities Request
enum {
    /** Success. **/
    USBTMC_STATUS_SUCCESS       = 0x01,
    /** Warning. This status is valid if a device has received a
      * USBTMC split transaction CHECK_STATUS request and the
      * request is still being processed. **/
    USBTMC_STATUS_PENDING       = 0x02,
    /**    0x03-0x1F Reserved Warning Reserved for USBTMC use. **/
    /**    0x20-0x3F Reserved Warning Reserved for subclass use. **/
    /**    0x40-0x7F Reserved Warning Reserved for VISA use. **/
    /** Failure. Unspecified reason, and a more specific
      * USBTMC_status is not defined. **/
    USBTMC_STATUS_FAILED        = 0x80,
    /** Failure. This status is only valid if a device has received
      * an INITIATE_ABORT_BULK_OUT or INITIATE_ABORT_BULK_IN
      * request and the specified transfer to abort is not
      * in progress. **/
    USBTMC_STATUS_TRANSFER_NOT_IN_PROGRESS  = 0x81,
    /** Failure. This status is valid if the device received a
     * CHECK_STATUS request and the device is not processing
     * an INITIATE request. **/
    USBTMC_STATUS_SPLIT_NOT_IN_PROGRESS     = 0x82,
    /** Failure. This status is valid if the device received a new
     * class-specific request and the device is still processing
     * an INITIATE. **/
    USBTMC_STATUS_SPLIT_IN_PROGRESS         = 0x83,
    /**    0x84-0x9F Reserved Failure Reserved for USBTMC use. **/
    /**    0xA0-0xBF Reserved Failure Reserved for subclass use. **/
    /**    0xC0-0xFF Reserved Failure Reserved for VISA use. **/
};

enum enum_MsgID
{
	//RESERVED = 0
	DEV_DEP_MSG_OUT 			= 1,
	REQUEST_DEV_DEP_MSG_IN 		= 2,
	DEV_DEP_MSG_IN 				= 2,
	//RESERVED
	VENDOR_SPECIFIC_OUT			= 126,
	REQUEST_VENDOR_SPECIFIC_IN	= 127,
	VENDOR_SPECIFIC_IN			= 127,
};


/**
    @typedef get_capabilities_response_t
    @brief Structure representing get capabilities response message.
 **/
typedef struct get_capabilities_response get_capabilities_response_t;

/**
 @name Get_Capabilities_Response
 @brief Get Capabilites Response
 */
//@{
struct get_capabilities_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;

	uint8_t reserved;

	/** BCD version number of the relevant USBTMC
	 *  specification for this USBTMC interface **/
	uint16_t bcdUSBTMC;

	/** USBTMC Interface capabilities
	 *
	 * Bit 2
	 * 1  The USBTMC interface accepts the INDICATOR_PULSE request.
	 * 0  The USBTMC interface does not accept the INDICATOR_PULSE request.
	 *
	 * Bit 1
	 * 1  The USBTMC interface is talk-only.
     * 0  The USBTMC interface is not talk-only.
	 *
	 * Bit 0
	 * 1  The USBTMC interface is listen-only.
	 * 0  The USBTMC interface is not listen-only.
	 * */
	uint8_t USBTMC_Interface_capabilities;

	/** USBTMC_Device_capabilities
	 *
	 * Bit 0
	 * 1  The device supports ending a Bulk-IN transfer from this USBTMC interface
	 * 		when a byte matches a specified TermChar.
	 * 0  The device does not support ending a Bulk-IN transfer from this USBTMC interface
	 * 		when a byte matches a specified TermChar.
	 *  **/
	uint8_t USBTMC_Device_capabilities;
	/** Reserved **/
	uint8_t reserved2[6];
	/** Reserved **/
	uint8_t reserved3 [12];

};

/**
    @typedef init_clear_response_t
    @brief Structure representing Initiate Clear response message.
 **/
typedef struct init_clear_response init_clear_response_t;

/**
 @name init_clear_response
 @brief Inititate Clear Response
 */
//@{
struct init_clear_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;

};

/**
    @typedef check_clear_response_t
    @brief Structure representing Check Clear response message.
 **/
typedef struct check_clear_response check_clear_response_t;

/**
 @name check_clear_response
 @brief Check Clear Response
 */
//@{
struct check_clear_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;
  uint8_t bmClear;

};

union transfer_size_u
{
    uint8_t array[4]; /// Array access
	struct
	{
        uint16_t lsw; /// Least significant word
		uint16_t msw; /// Most significant word

	} w; /// Word access
	struct
	{
        uint8_t lsb;  /// Least significant byte
        uint8_t mlsb; /// Middle Less significant byte
        uint8_t musb; /// Middle Upper significant byte
        uint8_t msb;  /// Most significant byte
	} b; /// Byte access
};

/**
    @typedef dev_dep_msg_in_t
    @brief Structure representing  dev dep msg out.
 **/
typedef struct dev_dep_msg_out dev_dep_msg_out_t;

/**
 @name dev_dep_msg_out
 @brief Device Dependent Message Out
 */
//@{
struct dev_dep_msg_out
{
	uint8_t	MsgId;
	uint8_t bTag;
	uint8_t bTagInverse;
	uint8_t reserved;
	union transfer_size_u TransferSize;
	uint8_t bmTransferAttributes;
	uint8_t reserverd2[3];
};

/**
    @typedef dev_dep_msg_in_t
    @brief Structure representing get dev_dep_msg in.
 **/
typedef struct dev_dep_msg_in dev_dep_msg_in_t;


/**
 @name dev_dep_msg_in
 @brief Device Dependent Message In
 */
//@{
struct dev_dep_msg_in
{
	uint8_t	MsgId;
	uint8_t bTag;
	uint8_t bTagInverse;
	uint8_t reserved;
	union transfer_size_u TransferSize;
	uint8_t bmTransferAttributes;
	uint8_t reserverd2[3];
};

/**
  @name : bmTransferAttributes TermChar Conditions Set
  @brief: Asserted if the conditions for termchar are set in Table 9.
 */
#define TMC_TA_AOTA_MASK 0x02 // who knows
/**
  @name : bmTransferAttributes EOM
  @brief: Asserted if the last message byte in the transfer is last byte of message.
 */
#define TMC_TA_EOM_MASK 0x01

/**
  @typedef: initiate_abort_response_t
  @brief: Structure representing response message for an Abort request.
 **/
typedef struct abort_bulk_response abort_bulk_response_t;

/**
  @name : Abort_Bulk_Response
  @brief: Response to an Abort request for a Bulk-IN or a Bulk-OUT
 */
//
struct abort_bulk_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;

	uint8_t bTag;
};

/**
  @typedef: check_abort_bulk_out_status_response_t
  @brief: Structure representing response message for a Check Abort Bulk Out Status request.
 **/
typedef struct check_abort_bulk_out_status_response check_abort_bulk_out_status_response_t;

/**
  @name : Check_Abort_Bulk_Out_Status_Response
  @brief: Response to a Check Abort Bulk Out Status request
 */
//
struct check_abort_bulk_out_status_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;
	uint8_t reserved[3];
	// Total number of bytes in the message received
    union transfer_size_u NBYTES_RXD;
};

/**
  @typedef: check_abort_bulk_in_status_response_t
  @brief: Structure representing response message for a Check Abort Bulk In Status request.
 **/
typedef struct check_abort_bulk_in_status_response check_abort_bulk_in_status_response_t;

/**
  @name : Check_Abort_Bulk_In Status_Response
  @brief: Response to a Check Abort Bulk In Status request
 */
//
struct check_abort_bulk_in_status_response
{
	/** Status indication for this request **/
	uint8_t USBTMC_status;
	uint8_t bmAbortBulkIn;
	uint16_t reserved;
	// Total number of bytes in the message received
    union transfer_size_u NBYTES_TXD;
};

/**
    @typedef   TMC_message_out_callback
    @brief     Callback declaration for TMC messages.
    @param[in] response Buffer to interpret command.
    @return    Unique application specific value for command.
 **/
typedef void (*TMC_message_out_callback)(char *command);

/**
    @typedef   TMC_request_message_in_callback
    @brief     Callback declaration for TMC messages.
    @param[out] response Buffer to write response to.
    @return    Number of bytes added to buffer.
 **/
typedef uint8_t (*TMC_request_message_in_callback)(char *response);

/**
    @typedef TMC_ctx
    @brief Struct containing callback functions for the TMC application
           layer.
 **/
typedef struct TMC_ctx TMC_ctx;

/**
    @struct TMC_ctx
    @brief Struct containing callback functions for the TMC application
           layer.
 **/
struct TMC_ctx
{
    /** Device Dependent Command callback function. **/
    TMC_message_out_callback message_cmd_cb;
    /** Device Dependent Message callback function. **/
    TMC_request_message_in_callback message_rsp_cb;
};

void tmc_initialise(TMC_ctx *ctx);

/**
 @brief      TMC Get_Capabilities Class Request
 @details    Host is requesting information about the TMC device
 **/
void tmc_class_req_capabilities(uint8_t devCaps, uint8_t interfaceCaps);

/**
 @brief      TMC Initiate Clear Class Request
 @details    Device must Halt the OUT endpoint and clear the IN and OUT buffers
 **/
void tmc_class_init_clear(void);

/**
 @brief    TMC Class Request check_clear_status
 @details  Check status on init_clear request
 **/
void tmc_class_check_clear(void);

/**
 @brief      TMC Abort_Bulk_Out Class Request
 @details    Host is aborting any current Bulk_Out requests
 **/
void tmc_class_init_abort_bulk_out(uint8_t bTag, uint8_t ep_num, uint8_t ep_dir);

/**
 @brief      TMC Check Abort_Bulk_Out Class Request
 @details    Host is requesting status on the previous Abort_Bulk_Out request
 **/
void tmc_class_check_abort_bulk_out(void);

/**
 @brief      TMC Abort_Bulk_In Class Request
 @details    Host is aborting any current Bulk_In requests
 **/
void tmc_class_init_abort_bulk_in(uint8_t bTag, uint8_t ep_num, uint8_t ep_dir);

/**
 @brief      TMC Check Abort_Bulk_In Class Request
 @details    Host is requesting status on the previous Abort_Bulk_In request
 **/
void tmc_class_check_abort_bulk_in(void);

/**
 @brief      TMC Process Requests
 @details    Process Message Out, Request Message In and Message In packets
 	 	 	 from the host. Callbacks in the application are used to start
 	 	 	 processing commands in a Message Out, and to format responses
 	 	 	 in a Request Message In.
 	 	 	 The formatted response is sent after a Request Message In and
 	 	 	 must complete before returning.
 	 	 	 This should be called periodically to handle the packets sent
 	 	 	 over the BULK endpoints of the TMC device.
 **/
void tmc_process(void);

#endif // __FT51_USB_TMC_H__